

library(data.table)


parse_date <- function(d) base::as.Date(d)

days_between <- function(a, b) as.integer(b - a) + 1

as.number <- function(x, default = 0, allow_inf = FALSE) { 
  y <- ifelse(is.null(x), default, x)
  y <- suppressWarnings(as.numeric(y))
  y <- ifelse(is.na(y), default, y)
  y <- ifelse(is.nan(y), default, y)
  y <- ifelse(is.infinite(y) & !allow_inf, default, y)
  y
}

get_rolling_stats <- function(accounts,
                              transactions,
                              today = Sys.Date(),
                              n_window = 90)
{
  r <- list(
    acct_sum = NA,
    n_txns = 0
  )
  if (!(is.list(accounts) && is.list(transactions))) {
    return(r)
  }
  if (!length(accounts)) {
    return(r)
  }
  accts <- as.data.frame(accounts)
  
  accts <- subset(as.data.frame(accounts), subtype %in% c('chck', 'save'))
  if (!nrow(accts)) {
    return(r)
  }
  r$acct_sum <- as.number(sum(as.numeric(accts$balances$current)), default = NA)
  
  ## parse
  transactions <- as.data.table(transactions)
  n_txns <- nrow(transactions)
  if (!n_txns || is.na(r$acct_sum)) {
    return(r)
  }
  txns <- subset(transactions, account_id %in% accts$account_id)
  txns <- txns[pending != TRUE, ]

  r$n_txns <- nrow(txns)
  r
}


get_stats <- function(body)
{
  accts <- as.list(body$transaction_data)$accounts
  txns <- as.list(body$transaction_data)$transactions
  r <- get_rolling_stats(accts, txns)
  r
}


test1 <- function()
{
  fn <- 'data.json'
  lines <- readLines(fn)
  
  data <- jsonlite::fromJSON(lines[[1]])

  for(n in 1:500) {
    r <- get_stats(data)
  }

}

test1()
